<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace PrometheusModule\Controller;


use App\Model\Table\ContainersTable;
use App\Model\Table\ServicesTable;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\AngularJS\Api;
use itnovum\openITCOCKPIT\Core\Views\ContainerPermissions;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use itnovum\openITCOCKPIT\Filter\GenericFilter;
use PrometheusModule\Model\Table\PrometheusAlertRulesTable;
use PrometheusModule\Model\Table\PrometheusExportersTable;

/**
 * Class PrometheusAlertRulesController
 * @package PrometheusModule\Controller
 */
class PrometheusAlertRulesController extends AppController {

    public function index() {
        if (!$this->isApiRequest()) {
            //Only ship template
            return;
        }

        /** @var PrometheusAlertRulesTable $PrometheusAlertRulesTable */
        $PrometheusAlertRulesTable = TableRegistry::getTableLocator()->get('PrometheusModule.PrometheusAlertRules');

        $GenericFilter = new GenericFilter($this->request);
        $GenericFilter->setFilters([
            'like' => [
                'PrometheusAlertRules.promql',
                'servicename',
            ],
            'equals' => [
                'PrometheusAlertRules.host_id'
            ]
        ]);

        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $GenericFilter->getPage());

        $MY_RIGHTS = [];
        if ($this->hasRootPrivileges === false) {
            $MY_RIGHTS = $this->MY_RIGHTS;
        }

        $all_alert_rules_tmp = $PrometheusAlertRulesTable->getAlertRulesIndex($GenericFilter, $PaginateOMat, $MY_RIGHTS);
        $all_alert_rules = [];
        foreach ($all_alert_rules_tmp as $alertRuleEntity) {
            $alertRule = $alertRuleEntity->toArray();
            if ($this->hasRootPrivileges) {
                $alertRule['allow_edit'] = true;
            } else {
                $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, explode(',', $alertRule['container_ids']));
                $allowEdit = $ContainerPermissions->hasPermission();

                $alertRule['allow_edit'] = $allowEdit;
            }

            $all_alert_rules[] = $alertRule;
        }

        $this->set('all_alert_rules', $all_alert_rules);
        $this->viewBuilder()->setOption('serialize', ['all_alert_rules']);
    }

    /**
     * @param int|null $id
     */
    public function edit($id = null) {

    }

    /**
     * @param int|null $id
     */
    public function delete($id = null) {

    }

}
