<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace PrometheusModule\Controller;

use App\Model\Entity\Changelog;
use App\Model\Entity\Host;
use App\Model\Table\ChangelogsTable;
use App\Model\Table\ContainersTable;
use App\Model\Table\HostsTable;
use App\Model\Table\HosttemplatesTable;
use App\Model\Table\ServicesTable;
use App\Model\Table\ServicetemplatesTable;
use Cake\Http\Exception\BadRequestException;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use GuzzleHttp\Exception\ClientException;
use itnovum\openITCOCKPIT\Core\AngularJS\Api;
use itnovum\openITCOCKPIT\Core\Comparison\ServiceComparisonForSave;
use itnovum\openITCOCKPIT\Core\FileDebugger;
use itnovum\openITCOCKPIT\Core\HostConditions;
use itnovum\openITCOCKPIT\Core\UUID;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\Service;
use itnovum\openITCOCKPIT\Filter\GenericFilter;
use itnovum\openITCOCKPIT\Filter\HostFilter;
use itnovum\openITCOCKPIT\Perfdata\UnitScaler;
use PrometheusModule\Form\PrometheusServiceForm;
use PrometheusModule\Lib\PrometheusApi;
use PrometheusModule\Model\Table\PrometheusAlertRulesTable;
use PrometheusModule\Model\Table\PrometheusExportersTable;


/**
 * Class PrometheusQueryController
 * @package PrometheusModule\Controller
 */
class PrometheusQueryController extends AppController {

    public function index() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }

        $hostId = $this->request->getQuery('hostId', 0);
        $filter = $this->request->getQuery('filter', []);
        $selected = $this->request->getQuery('selected', []);
        if (!is_array($filter)) {
            $filter = [];
        }
        if (!is_array($selected)) {
            $selected = [];
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if (!$HostsTable->existsById($hostId)) {
            throw new NotFoundException(__('Invalid host'));
        }

        /** @var Host $host */
        $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
        if (!$this->allowedByContainerId($host->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        $PrometheusApi = new PrometheusApi();
        $targets = $PrometheusApi->queryingTargetMetadata($host->get('uuid'), $filter, $selected);

        $this->set('targets', $targets);
        $this->set('host', $host);
        $this->viewBuilder()->setOption('serialize', ['targets', 'host']);

    }

    public function toService() {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }
    }

    /**
     * @param int|null $serviceId
     */
    public function editService($serviceId = null) {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }

        /** @var PrometheusAlertRulesTable $PrometheusAlertRulesTable */
        $PrometheusAlertRulesTable = TableRegistry::getTableLocator()->get('PrometheusModule.PrometheusAlertRules');
        /** @var ContainersTable $ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');
        /** @var HosttemplatesTable $HosttemplatesTable */
        $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var ServicetemplatesTable $ServicetemplatesTable */
        $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        if ($this->request->is('POST')) {

            $servicetemplateId = $this->request->getData('Service.servicetemplate_id');
            if ($servicetemplateId === null) {
                throw new \Exception('Service.servicetemplate_id needs to set.');
            }

            if (!$ServicetemplatesTable->existsById($servicetemplateId)) {
                throw new NotFoundException(__('Invalid service template'));
            }
            $servicetemplate = $ServicetemplatesTable->getServicetemplateForDiff($servicetemplateId);

            if ($this->request->getData('Service.prometheus_alert_rule.id') === null) {
                throw new \Exception('Service.prometheus_alert_rule.id needs to set.');
            }

            $host = $HostsTable->get($this->request->getData('Service.host_id', 0));

            $service = $ServicesTable->getServiceForEdit($this->request->getData('Service.id', 0));
            $serviceForChangelog = $service;

            $servicename = $this->request->getData('Service.name');
            if ($servicename === null || $servicename === '') {
                $servicename = $servicetemplate['Servicetemplate']['name'];
            }

            $ServiceComparisonForSave = new ServiceComparisonForSave(
                $this->request->getData(),
                $servicetemplate,
                $HostsTable->getContactsAndContactgroupsById($host->id),
                $HosttemplatesTable->getContactsAndContactgroupsById($host->hosttemplate_id)
            );
            $dataForSave = $ServiceComparisonForSave->getDataForSaveForAllFields();

            //Add required fields for validation
            $dataForSave['servicetemplate_flap_detection_enabled'] = $servicetemplate['Servicetemplate']['flap_detection_enabled'];
            $dataForSave['servicetemplate_flap_detection_on_ok'] = $servicetemplate['Servicetemplate']['flap_detection_on_ok'];
            $dataForSave['servicetemplate_flap_detection_on_warning'] = $servicetemplate['Servicetemplate']['flap_detection_on_warning'];
            $dataForSave['servicetemplate_flap_detection_on_critical'] = $servicetemplate['Servicetemplate']['flap_detection_on_critical'];
            $dataForSave['servicetemplate_flap_detection_on_unknown'] = $servicetemplate['Servicetemplate']['flap_detection_on_unknown'];

            $dataForSave['service_type'] = PROMETHEUS_SERVICE;

            //Update contact data
            $serviceEntity = $ServicesTable->get($this->request->getData('Service.id', 0));
            $serviceEntity->setAccess('uuid', false);
            $serviceEntity->setAccess('id', false);
            $serviceEntity->setAccess('host_id', false);

            $serviceEntity = $ServicesTable->patchEntity($serviceEntity, $dataForSave, [
                'associated' => [
                    'PrometheusAlertRules' => [
                        'accessibleFields' => [
                            'id' => true
                        ]
                    ]
                ]
            ]);
            $ServicesTable->save($serviceEntity);

            $request = $this->request->getData();
            $request['Host'] = [
                [
                    'id'   => $host->id,
                    'name' => $host->name
                ]
            ];

            if ($serviceEntity->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $serviceEntity->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            } else {
                //No errors

                $User = new User($this->getUser());

                $extDataForChangelog = $ServicesTable->resolveDataForChangelog($request);

                /** @var  ChangelogsTable $ChangelogsTable */
                $ChangelogsTable = TableRegistry::getTableLocator()->get('Changelogs');

                $changelog_data = $ChangelogsTable->parseDataForChangelog(
                    'edit',
                    'services',
                    $serviceEntity->get('id'),
                    OBJECT_SERVICE,
                    $host->container_id,
                    $User->getId(),
                    $host->name . '/' . $servicename,
                    array_merge($ServicesTable->resolveDataForChangelog($request), $request),
                    array_merge($ServicesTable->resolveDataForChangelog($serviceForChangelog), $serviceForChangelog)
                );
                if ($changelog_data) {
                    /** @var Changelog $changelogEntry */
                    $changelogEntry = $ChangelogsTable->newEntity($changelog_data);
                    $ChangelogsTable->save($changelogEntry);
                }

                if ($this->isJsonRequest()) {
                    $this->serializeCake4Id($serviceEntity); // REST API ID serialization
                    return;
                }
            }
            $this->set('service', $serviceEntity);
            $this->viewBuilder()->setOption('serialize', ['service']);

        }

        if (!$PrometheusAlertRulesTable->existsByServiceId($serviceId)) {
            throw new NotFoundException(__('Invalid alert rule'));
        }

        $alertRule = $PrometheusAlertRulesTable->getAlertRuleServiceByIdForEdit($serviceId);

        $containerIdsToCheck = Hash::extract($alertRule, 'host.hosts_to_containers_sharing.{n}.id');
        $containerIdsToCheck[] = $alertRule['host']['container_id'];
        if (!$this->allowedByContainerId($containerIdsToCheck, true)) {
            $this->render403();
            return;
        }

        $containerId = $ContainersTable->resolveChildrenOfContainerIds($alertRule['host']['container_id']);
        $servicetemplates = $ServicetemplatesTable->getServicetemplatesByContainerId($containerId, 'list', PROMETHEUS_SERVICE);
        $servicetemplates = Api::makeItJavaScriptAble($servicetemplates);

        $serviceName = $alertRule['service']['name'];
        if ($serviceName === null || $serviceName === '') {
            $serviceName = $alertRule['service']['servicetemplate']['name'];
        }


        $postData = [
            'Service' => [
                'id'                 => $alertRule['service_id'],
                'host_id'            => $alertRule['host_id'],
                'servicetemplate_id' => $alertRule['service']['servicetemplate_id'],
                'name'               => $serviceName,

                'prometheus_alert_rule' => [
                    'id'                 => $alertRule['id'],
                    'host_id'            => $alertRule['host_id'],
                    'service_id'         => $alertRule['service_id'],
                    'servicetemplate_id' => $alertRule['service']['servicetemplate_id'],
                    'promql'             => $alertRule['promql'],
                    'unit'               => $alertRule['unit'],
                    'threshold_type'     => $alertRule['threshold_type'],
                    'warning_operator'   => $alertRule['warning_operator'],
                    'warning_min'        => $alertRule['warning_min'],
                    'warning_max'        => $alertRule['warning_max'],
                    'warning_longer_as'  => $alertRule['warning_longer_as'],
                    'critical_operator'  => $alertRule['critical_operator'],
                    'critical_min'       => $alertRule['critical_min'],
                    'critical_max'       => $alertRule['critical_max'],
                    'critical_longer_as' => $alertRule['critical_longer_as']
                ]
            ]
        ];

        $selectedMetrics = [];
        preg_match_all('/([\w]+)(?:{)/', $alertRule['promql'], $matches);
        if (!empty($matches)) {
            foreach ($matches as $match) {
                foreach ($match as $line) {
                    if (strpos($line, '{') === false) {
                        $selectedMetrics[] = $line;
                    }
                }
            }
        }

        $this->set('postData', $postData);
        $this->set('host', $alertRule['host']);
        $this->set('servicetemplates', $servicetemplates);
        $this->set('selectedMetrics', $selectedMetrics);

        $this->viewBuilder()->setOption('serialize', ['postData', 'servicetemplates', 'host', 'selectedMetrics']);
    }

    /**
     * This method is used by /services/browser to load and display
     * the current Prometheus status
     * @param string|int $serviceId
     */
    public function browser_directive($serviceId = null) {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }

        /** @var $HostsTable HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var $ServicesTable ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        /** @var PrometheusAlertRulesTable $PrometheusAlertRulesTable */
        $PrometheusAlertRulesTable = TableRegistry::getTableLocator()->get('PrometheusModule.PrometheusAlertRules');

        if (!$ServicesTable->existsById($serviceId)) {
            throw new NotFoundException(__('Service not found'));
        }

        $alertRule = $PrometheusAlertRulesTable->getAlertRuleWithServiceByServiceId($serviceId);

        //Check permissions
        $host = $HostsTable->getHostForServiceEdit($alertRule->get('service')->get('host_id'));
        if (!$this->allowedByContainerId($host['Host']['hosts_to_containers_sharing']['_ids'], false)) {
            $this->render403();
            return;
        }


        $PrometheusApi = new PrometheusApi();

        try {
            $metricValue = $PrometheusApi->getValueByMetric($alertRule['promql']);

            $this->set('metricValue', $metricValue);
            $this->set('alertRule', $alertRule);
            $this->viewBuilder()->setOption('serialize', ['metricValue', 'alertRule']);
        } catch (ClientException $e) {

            $json = $e->getResponse()->getBody()->getContents();
            $json = json_decode($json, true);

            $this->response = $this->response->withStatus(400);
            $this->set('error', $json['error'] ?? __('Unknown error'));
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

    }

    /****************************
     *       AJAX METHODS       *
     ****************************/

    public function getPerfdataByUuid() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $hostUuid = $this->request->getQuery('host_uuid', '');
        $metric = $this->request->getQuery('metric', '');
        $hours = $this->request->getQuery('hours', '');
        $start = (int)$this->request->getQuery('start', '');
        $end = (int)$this->request->getQuery('end', '');
        $jsTimestamp = (bool)$this->request->getQuery('jsTimestamp', 0);
        $scale = $this->request->getQuery('scale', 'true') === 'true';
        $promql = $this->request->getQuery('promql', null);

        if (is_numeric($hours)) {
            $hours = (int)$hours;
            $start = time() - ($hours * 3600);
            $end = time();
        }

        if ($start === null || $end === null) {
            $start = time() - (3 * 3600);
            $end = time();
        }

        try {
            $PrometheusApi = new PrometheusApi();

            if ($promql === null) {
                $tmpPerfData = $PrometheusApi->getPerfdataByUuidAndMetric($hostUuid, $metric, $start, $end);
            } else {
                $tmpPerfData = $PrometheusApi->getPerfdataByPromQL($promql, $start, $end);
            }

            $gauge = [];
            if (isset($tmpPerfData['status']) && isset($tmpPerfData['data'])) {
                if ($tmpPerfData['status'] === "success") {
                    if (isset($tmpPerfData['data']['result'][0]['values'])) {
                        foreach ($tmpPerfData['data']['result'][0]['values'] as $record) {
                            if ($jsTimestamp) {
                                $gauge[$record[0] * 1000] = floatval($record[1]);
                            } else {
                                $gauge[$record[0]] = floatval($record[1]);
                            }
                        }
                    }
                }
            }

            $metricUnit = $PrometheusApi->getUnitByMetricName($metric);

            //Make it look like Nagios performance data...
            $performance_data =
                [
                    [
                        'datasource' => [
                            'ds'    => $metric,
                            'name'  => $metric,
                            'label' => $metric,
                            'unit'  => $metricUnit,
                            'warn'  => null,
                            'crit'  => null,
                            'min'   => null,
                            'max'   => null,
                        ],
                        'data'       => $gauge
                    ]
                ];

            if ($scale === true) {
                foreach ($performance_data as $index => $gauge) {
                    $UnitScaler = new UnitScaler($gauge);
                    $performance_data[$index] = $UnitScaler->scale();
                }
            }

            $this->set('performance_data', $performance_data);
            $this->viewBuilder()->setOption('serialize', ['performance_data']);

        } catch (\Exception $e) {
            error_log($e->getMessage());
            $performance_data = [];
        }


        $this->set('performance_data', $performance_data);
        $this->viewBuilder()->setOption('serialize', ['performance_data']);

    }

    public function loadCurrentValueByMetric() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }

        $hostUuid = $this->request->getQuery('hostUuid', null);
        $metric = $this->request->getQuery('metric', null);

        $PrometheusApi = new PrometheusApi();
        $metricDetails = $PrometheusApi->getCurrentValueByMetric($hostUuid, $metric);

        $this->set('metricDetails', $metricDetails);
        $this->viewBuilder()->setOption('serialize', ['metricDetails']);
    }

    public function loadValueByMetric() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }

        $metric = $this->request->getQuery('metric', null);

        $PrometheusApi = new PrometheusApi();

        try {
            $metricValue = $PrometheusApi->getValueByMetric($metric);
        } catch (ClientException $e) {

            $json = $e->getResponse()->getBody()->getContents();
            $json = json_decode($json, true);

            $this->response = $this->response->withStatus(400);
            $this->set('error', $json['error'] ?? __('Unknown error'));
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        $this->set('metricValue', $metricValue);
        $this->viewBuilder()->setOption('serialize', ['metricValue']);
    }

    /**
     * @param bool $onlyHostsWithWritePermission
     */
    public function loadHostsByString($onlyHostsWithWritePermission = false) {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $selected = $this->request->getQuery('selected');
        $includeDisabled = $this->request->getQuery('includeDisabled') === 'true';

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var PrometheusExportersTable $PrometheusExportersTable */
        $PrometheusExportersTable = TableRegistry::getTableLocator()->get('PrometheusModule.PrometheusExporters');

        $HostFilter = new HostFilter($this->request);

        $HostCondition = new HostConditions($HostFilter->ajaxFilter());
        $HostCondition->setIncludeDisabled($includeDisabled);
        $HostCondition->setContainerIds($this->MY_RIGHTS);
        if ($onlyHostsWithWritePermission) {
            $writeContainers = [];
            foreach ($this->MY_RIGHTS_LEVEL as $containerId => $rightLevel) {
                $rightLevel = (int)$rightLevel;
                if ($rightLevel === WRITE_RIGHT) {
                    $writeContainers[$containerId] = $rightLevel;
                }
            }
            $HostCondition->setContainerIds(array_keys($writeContainers));
        }

        $hosts = Api::makeItJavaScriptAble(
            $PrometheusExportersTable->getPrometheusHostsForAngular($HostCondition, $selected, true)
        );

        $this->set('hosts', $hosts);
        $this->viewBuilder()->setOption('serialize', ['hosts']);
    }

    public function validateService() {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        $service = $this->request->getData('Service', []);

        $PrometheusServiceForm = new PrometheusServiceForm();
        $PrometheusServiceForm->execute($service);

        if (!empty($PrometheusServiceForm->getErrors())) {
            $this->response = $this->response->withStatus(400);
            $this->set('success', false);
            $this->set('error', $PrometheusServiceForm->getErrors());
            $this->viewBuilder()->setOption('serialize', ['success', 'error']);
            return;
        }

        $this->set('success', true);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    /**
     * @param int|null $containerId
     */
    public function loadServicetemplates($hostId = null) {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var ContainersTable $ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');
        /** @var ServicetemplatesTable $ServicetemplatesTable */
        $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');

        if (!$HostsTable->existsById($hostId)) {
            throw new NotFoundException(__('Invalid host id'));
        }

        $host = $HostsTable->get($hostId);

        $containerId = $host->get('container_id');

        $containerId = $ContainersTable->resolveChildrenOfContainerIds($containerId);
        $servicetemplates = $ServicetemplatesTable->getServicetemplatesByContainerId($containerId, 'list', PROMETHEUS_SERVICE);
        $servicetemplates = Api::makeItJavaScriptAble($servicetemplates);

        $this->set('servicetemplates', $servicetemplates);
        $this->viewBuilder()->setOption('serialize', ['servicetemplates']);
    }

    public function createService() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        $servicetemplateId = $this->request->getData('Service.servicetemplate_id');

        if (!$this->request->getData('Service.host_id') || !$this->request->getData('Service.servicetemplate_id')) {
            $errors = [];
            $this->response = $this->response->withStatus(400);
            if (!$this->request->getData('Service.host_id')) {

                $errors['host_id'] = [
                    'empty' => __('This field cannot be left empty')
                ];
            }
            if (!$this->request->getData('Service.servicetemplate_id')) {
                $errors['servicetemplate_id'] = [
                    'empty' => __('This field cannot be left empty')
                ];
            }
            $this->set('error', $errors);
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        if ($servicetemplateId === null) {
            throw new BadRequestException('Service.servicetemplate_id needs to set.');
        }

        $hostId = $this->request->getData('Service.host_id');
        if ($hostId === null) {
            throw new BadRequestException('Service.host_id needs to set.');
        }

        /** @var $HosttemplatesTable HosttemplatesTable */
        $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');
        /** @var $HostsTable HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var $ServicetemplatesTable ServicetemplatesTable */
        $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');
        /** @var $ServicesTable ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        if (!$ServicetemplatesTable->existsById($servicetemplateId)) {
            throw new NotFoundException(__('Invalid service template'));
        }

        $host = $HostsTable->get($hostId);
        $request = $this->request->getData();
        $request['Host'] = [
            'id'   => $host->get('id'),
            'name' => $host->get('name')
        ];

        $servicetemplate = $ServicetemplatesTable->getServicetemplateForDiff($servicetemplateId);


        $servicename = $this->request->getData('Service.name');
        if ($servicename === null || $servicename === '') {
            $servicename = $servicetemplate['Servicetemplate']['name'];
        }

        $ServiceComparisonForSave = new ServiceComparisonForSave(
            $request,
            $servicetemplate,
            $HostsTable->getContactsAndContactgroupsById($host->get('id')),
            $HosttemplatesTable->getContactsAndContactgroupsById($host->get('hosttemplate_id'))
        );
        $serviceData = $ServiceComparisonForSave->getDataForSaveForAllFields();
        $serviceData['uuid'] = UUID::v4();

        //Add required fields for validation
        $serviceData['servicetemplate_flap_detection_enabled'] = $servicetemplate['Servicetemplate']['flap_detection_enabled'];
        $serviceData['servicetemplate_flap_detection_on_ok'] = $servicetemplate['Servicetemplate']['flap_detection_on_ok'];
        $serviceData['servicetemplate_flap_detection_on_warning'] = $servicetemplate['Servicetemplate']['flap_detection_on_warning'];
        $serviceData['servicetemplate_flap_detection_on_critical'] = $servicetemplate['Servicetemplate']['flap_detection_on_critical'];
        $serviceData['servicetemplate_flap_detection_on_unknown'] = $servicetemplate['Servicetemplate']['flap_detection_on_unknown'];

        $serviceData['service_type'] = PROMETHEUS_SERVICE;

        $service = $ServicesTable->newEntity($serviceData);

        $ServicesTable->save($service);
        if ($service->hasErrors()) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', $service->getErrors());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        } else {
            //No errors

            $User = new User($this->getUser());

            $extDataForChangelog = $ServicesTable->resolveDataForChangelog($request);
            /** @var  ChangelogsTable $ChangelogsTable */
            $ChangelogsTable = TableRegistry::getTableLocator()->get('Changelogs');

            $changelog_data = $ChangelogsTable->parseDataForChangelog(
                'add',
                'services',
                $service->get('id'),
                OBJECT_SERVICE,
                $host->get('container_id'),
                $User->getId(),
                $host->get('name') . '/' . $servicename,
                array_merge($request, $extDataForChangelog)
            );

            if ($changelog_data) {
                /** @var Changelog $changelogEntry */
                $changelogEntry = $ChangelogsTable->newEntity($changelog_data);
                $ChangelogsTable->save($changelogEntry);
            }


            if ($this->isJsonRequest()) {
                $this->serializeCake4Id($service); // REST API ID serialization
                return;
            }
        }
        $this->set('service', $service);
        $this->viewBuilder()->setOption('serialize', ['service']);


    }
}
