angular.module('openITCOCKPIT')
    .controller('PrometheusAlertRulesIndexController', function($scope, $http, SortService, MassChangeService, QueryStringService, $stateParams){

        SortService.setSort(QueryStringService.getStateValue($stateParams, 'sort', 'servicename'));
        SortService.setDirection(QueryStringService.getStateValue($stateParams, 'direction', 'asc'));
        $scope.currentPage = 1;
        $scope.useScroll = true;

        $scope.init = true;
        $scope.hostId = null;

        if($stateParams.hostId !== null){
            $scope.hostId = parseInt($stateParams.hostId, 10);
        }

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                PrometheusAlertRules: {
                    promql: '',
                },
                servicename: ''
            };
        };
        defaultFilter();
        /*** Filter end ***/

        $scope.massChange = {};
        $scope.selectedElements = 0;
        $scope.deleteUrl = '/services/delete/';

        $scope.loadTimezone = function(){
            $http.get("/angular/user_timezone.json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.timezone = result.data.timezone;
            });
        };

        $scope.loadHosts = function(searchString){
            $http.get("/prometheus_module/PrometheusQuery/loadHostsByString.json", {
                params: {
                    'angular': true,
                    'filter[Hosts.name]': searchString,
                    //'selected[]': $scope.hostId
                }
            }).then(function(result){
                $scope.hosts = result.data.hosts;
                $scope.init = false;
            });
        };

        $scope.load = function(){
            if($scope.hostId === null){
                return;
            }

            $http.get("/prometheus_module/PrometheusAlertRules/index.json", {
                params: {
                    'angular': true,
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'filter[PrometheusAlertRules.host_id]': $scope.hostId,
                    'filter[PrometheusAlertRules.promql]': $scope.filter.PrometheusAlertRules.promql,
                    'filter[servicename]': $scope.filter.servicename
                }
            }).then(function(result){
                $scope.alert_rules = result.data.all_alert_rules;
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;
            });
        };

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
            $scope.undoSelection();
        };

        $scope.selectAll = function(){
            if($scope.alert_rules){
                for(var key in $scope.alert_rules){
                    if($scope.alert_rules[key].allow_edit)
                        var serviceId = $scope.alert_rules[key]._matchingData.PrometheusAlertRules.service_id;
                    $scope.massChange[serviceId] = true;
                }
            }
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.getObjectsForDelete = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.alert_rules){
                for(var serviceId in selectedObjects){
                    if(serviceId == $scope.alert_rules[key]._matchingData.PrometheusAlertRules.service_id){
                        objects[serviceId] =
                            $scope.alert_rules[key].Hosts.name + '/' +
                            $scope.alert_rules[key].servicename;
                    }

                }
            }
            return objects;
        };

        $scope.getObjectForDelete = function(alertRule){
            var object = {};
            object[alertRule._matchingData.PrometheusAlertRules.service_id] = alertRule.Hosts.name + '/' + alertRule.servicename;
            return object;
        };


        $scope.changepage = function(page){
            $scope.undoSelection();
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        $scope.$watch('hostId', function(){
            $scope.load();
        });

        //Fire on page load
        $scope.loadTimezone();
        $scope.loadHosts('');
        SortService.setCallback($scope.load);

        $scope.$watch('filter', function(){
            if($scope.init){
                return;
            }

            $scope.undoSelection();
            $scope.load();
        }, true);


        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);
    });
