angular.module('openITCOCKPIT')
    .controller('PrometheusQueryIndexController', function($scope, $http, SortService, MassChangeService, $stateParams){

        $scope.init = true;
        $scope.hostId = null;
        if($stateParams.hostId !== null){
            $scope.hostId = parseInt($stateParams.hostId, 10);
        }

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                PrometheusQuery: {
                    metric: '',
                }
            };
        };
        defaultFilter();
        /*** Filter end ***/
        $scope.massChange = {};
        $scope.selectedElements = 0;

        $scope.loadTimezone = function(){
            $http.get("/angular/user_timezone.json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.timezone = result.data.timezone;
            });
        };

        $scope.loadHosts = function(searchString){
            $http.get("/prometheus_module/PrometheusQuery/loadHostsByString.json", {
                params: {
                    'angular': true,
                    'filter[Hosts.name]': searchString,
                    //'selected[]': $scope.hostId
                }
            }).then(function(result){
                $scope.hosts = result.data.hosts;
                $scope.init = false;
            });
        };

        $scope.load = function(){
            if($scope.hostId === null){
                return;
            }

            $http.get("/prometheus_module/PrometheusQuery/index.json", {
                params: {
                    'angular': true,
                    'hostId': $scope.hostId,
                    'filter[PrometheusQuery.name]': $scope.filter.PrometheusQuery.metric,
                }
            }).then(function(result){
                $scope.host = result.data.host;

                var metrics = [];

                if(result.data.targets.hasOwnProperty('status')){
                    if(result.data.targets.status === 'success'){
                        for(var index in result.data.targets.data){
                            metrics.push(result.data.targets.data[index]);
                        }
                    }
                }

                $scope.metrics = metrics;
            });
        };

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
            $scope.undoSelection();
        };

        $scope.selectAll = function(){
            if($scope.metrics){
                for(var key in $scope.metrics){
                    var metricName = $scope.metrics[key].metric;
                    $scope.massChange[metricName] = true;
                }
            }
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.metricsForServiceAdd = function(){
            if($scope.init){
                return;
            }
            var metrics = MassChangeService.getSelected();
            return Object.keys(metrics);
        };

        $scope.getCurrentMetricForServiceAdd = function(metric){
            return [metric.metric];
        };

        $scope.$watch('hostId', function(){
            $scope.load();
        });

        //Fire on page load
        $scope.loadTimezone();
        $scope.loadHosts('');

        $scope.$watch('filter', function(){
            if($scope.init){
                return;
            }

            $scope.undoSelection();
            $scope.load();
        }, true);


        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);
    });
